<?php

namespace App\Traits;

use App\Helpers\FormatHelper;
use App\Mail\SendMail;
use App\Repositories\CommonRepo;
use App\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use Validator;
use Illuminate\Support\Facades\Hash;


//use Yajra\DataTables\Contracts\DataTable;

trait UserLoginTrait
{
    //use RedirectsUsers, ThrottlesLogins;
    //use CommonTrait;

    public function loginUser($request)
    {
        $input = $request->all();

        $validator = $this->validateLogin($request);
        if ($validator->fails()) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'step' => 0,
                'message' => $validator->messages()->toArray()
            ];

            return $json;
        }

        $user = $this->userRepo->checkUserByUserName($input['email']);

        if (!empty($user) && count($user) > 0 && $user[0]->status == 0) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'step' => 0,
                'message' => 'This user has inactivated.'
            ];

            return $json;
        }

        if ($this->checkUserValid($input, $user)) {

            $layers = CommonRepo::getLayers();

            if ($layers[0]->status == 1) {

                if (!FormatHelper::checkSecure()) {
                    $json = [
                        'type' => 'error',
                        'redirect' => 'reload',
                        'step' => 0,
                        'message' => 'Your SSL certificate is invalid.'
                    ];

                    return $json;
                }
            }

            $stepTwo = FormatHelper::checkStepTwo($layers);

            if ($stepTwo) {
                $json = [
                    'type' => 'success',
                    'redirect' => '',
                    'step' => 2,
                    'layers' => FormatHelper::getLayers(),
                    'message' => ''
                ];
            } else {
                $stepThree = FormatHelper::checkStepThree($layers);

                // check email verification exist
                if ($stepThree) {
                    // send email verification code
                    $json = $this->sendCodeEmail($input);
                } else {
                    $credentials = $request->only('email', 'password');

                    if ($this->tryLogin($credentials)) {
                        $json = [
                            'type' => 'success',
                            'redirect' => url('/'),
                            'step' => 4,
                            'message' => 'Successfully logged in.'
                        ];

                        return $json;
                    }

                    $json = [
                        'type' => 'error',
                        'redirect' => '',
                        'step' => 0,
                        'message' => 'Sorry! Unable to find a user with given credentials.'
                    ];
                }
            }

            return $json;
        }

        $json = [
            'type' => 'error',
            'redirect' => '',
            'step' => 0,
            'message' => 'Please check your credentials.'
        ];

        return $json;

    }

    protected function tryLogin($credentials)
    {
        $credentials['deleted_at'] = NULL;
        if (Auth::attempt($credentials)) {
            CommonRepo::commonUpdate('App\User', ['id' => Auth::id()], ['is_logged_in' => 1]);
            return true;
        }
        return false;
    }

    public function checkUserValid($input, $user)
    {
        if (!empty($user) && count($user) > 0) {
            return $this->hasher->check($input['password'], $user[0]->password);
        }

        return false;
    }

    /**
     * Validate the user login request.
     *
     * @param  \Illuminate\Http\Request $request
     * @return void
     */
    protected function validateLogin($request)
    {
        $input = $request->all();
        $rules = [
            'email' => 'required|string',
            'password' => 'required|string',
        ];

        return Validator::make($input, $rules);
    }

    protected function getLoginStep()
    {
        return User::select('step')->where('id', Auth::id())->get();
    }

    protected function loginUserStepTwo($request)
    {
        $input = $request->all();
        //dd($input);
        $layers = FormatHelper::getLayers();

        $validator = $this->validateStepTwo($request, $layers);
        if ($validator->fails()) {
            //dd($validator->messages()->toArray());
            $json = [
                'type' => 'error',
                'redirect' => '',
                'current_step' => 2,
                'message' => $validator->messages()->toArray()
            ];

            return $json;
        }

        $result = $this->checkStepTwo($input, $layers);

        // if one not match send error
        if (count($result) > 0) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'current_step' => 2,
                'message' => $result
            ];

            return $json;
        }

        /**
         * check email verification exist
         */
        if ($layers['dynamic_code'] == 1) {
            return $this->sendCodeEmail($input);
        }

        $credentials = $request->only('email', 'password');

        if ($this->tryLogin($credentials)) {
            $json = [
                'type' => 'success',
                'redirect' => url('/'),
                'step' => 4,
                'message' => 'Successfully logged in.'
            ];

            return $json;
        }


    }

    /**
     * Check user pin, mac and ip address valid
     * @param $input
     * @param $layers
     * @return array
     *
     */
    protected function checkStepTwo($input, $layers)
    {
        $message = [];
        $userData = $this->userRepo->checkUserByUserName($input['email']);
        //dd($userData);
        //echo $userData->pin_number;exit;

        if ($layers['pin'] == 1) {
            // if user doesn't have pin number
            if ($userData[0]->pin_number == "") {
                $message['pin_number'] = [0 => 'Please assign a pin number for this user.'];
            } else {
                if ($userData[0]->pin_number != $input['pin_number']) {
                    $message['pin_number'] = [0 => 'The pin number is invalid.'];
                }
            }
        }

        if ($layers['mac'] == 1) {

            // if user doesn't have MAC addreess
            if ($userData[0]->mac_address == "") {
                $message['mac_address'] = [0 => 'Please assign a mac addreess for this user.'];
            } else {

                $pcMac = $this->getMAC();
                if ($input['mac_address'] != $pcMac) {
                    $message['mac_address'] = [0 => 'The mac addreess is invalid.'];
                }
                if ($userData[0]->mac_address != $input['mac_address']) {
                    $message['mac_address'] = [0 => 'The mac addreess is invalid.'];
                }
            }
        }

        if ($layers['ip_address'] == 1) {

            // if user doesn't have IP addreess
            if ($userData[0]->ip_address == "") {
                $message['ip_address'] = [0 => 'Please assign a IP addreess for this user.'];
            } else {
                // TODO get local ip and check both are same
                if ($userData[0]->ip_address != $input['ip_address']) {
                    $message['ip_address'] = [0 => 'The ip adress is invalid.'];
                }
            }
        }

        return $message;
    }


    protected function logOutUser()
    {

    }

    protected function validateStepTwo($request, $layers)
    {
        $input = $request->all();
        $rules = [];

        if ($layers['pin'] == 1) {
            $rules['pin_number'] = 'required|string';
        }

        if ($layers['mac'] == 1) {
            $rules['mac_address'] = 'required|string';
        }

        if ($layers['ip_address'] == 1) {
            $rules['ip_address'] = 'required|string';
        }

        return Validator::make($input, $rules);
    }

    protected function getMAC()
    {
        ob_start();
        system('getmac');
        $Content = ob_get_contents();
        ob_clean();
        return substr($Content, strpos($Content, '\\') - 20, 17);
    }

    protected function sendCodeEmail($input)
    {
        $code = FormatHelper::randNumberGenerator(6);
        $token = md5(str_random(20));
        $user = $this->userRepo->checkUserByUserName($input['email']);
        $time = Carbon::parse()->toDateTimeString();
        $dataArray = [
            'user_id' => $user[0]->id,
            'login_code' => $code,
            'email_token' => $token,
            'created_at' => $time,
        ];

        $this->userRepo->addLoginCode($dataArray);

        $data = view('admin.security.emails.auth.verification_code', compact('input', 'user', 'code'));
        $subject = "User verification code";
        $message = 'Email verification code sent.Check your email.';

        // send email with verification code
        return $this->sendCommonMail($input['email'], $data, $subject, $message);
    }

    public function loginUserStepThree($request)
    {
        $input = $request->all();

        $validator = $this->validateStepThree($request);
        if ($validator->fails()) {

            $json = [
                'type' => 'error',
                'redirect' => '',
                'current_step' => 2,
                'message' => $validator->messages()->toArray()
            ];

            return $json;
        }

        $user = $this->userRepo->checkUserByUserName($input['email']);
        $result = $this->checkStepThree($input, $user);

        // if one not match send error
        if (count($result) > 0) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'current_step' => 3,
                'message' => $result
            ];

            return $json;
        }

        $credentials = $request->only('email', 'password');
        if ($this->tryLogin($credentials)) {

            $where = ['user_id' => $user[0]->id];
            $data = ['status' => 0];
            CommonRepo::commonUpdate('App\Models\LoginCode', $where, $data);

            $json = [
                'type' => 'success',
                'redirect' => url('/'),
                'step' => 4,
                'message' => 'Successfully logged in.'
            ];

            return $json;
        }
    }

    protected function validateStepThree($request)
    {
        $input = $request->all();
        $rules = [
            'email_code' => 'required|string|max:6'
        ];

        return Validator::make($input, $rules);
    }

    protected function checkStepThree($input, $user)
    {
        //dd($input);
        $message = [];
        $time = Carbon::parse('-5 minutes')->toDateTimeString();
        //dd($time);
        $codeInfo = $this->userRepo->getLoginCodeDataByUser($user[0]->id, $time, $input['email_code']);

        if (!empty($codeInfo) && count($codeInfo) > 0) {

            if ($input['email_code'] != $codeInfo[0]->login_code) {
                $message['email_code'] = [0 => 'The email code is invalid.'];
            }
        } else {
            $message['no_data'] = [0 => 'The email code is invalid.'];
        }

        return $message;
    }

    protected function passwordReset($request)
    {
        $input = $request->all();
        $validator = $this->validateResetPassword($request);
        if ($validator->fails()) {

            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => $validator->messages()->toArray()
            ];

            return $json;
        }

        $tokenValid = $this->getResetInfo($input['token']);

        if ($tokenValid) {
            $model = 'App\User';
            $whereArray = ['email' => $tokenValid[0]->email];
            $pwd = Hash::make($input['password']);
            $data = [
                'password' => $pwd,
                'status' => 1,
            ];

            CommonRepo::commonUpdate($model, $whereArray, $data);

            CommonRepo::commonDelete('App\Models\PasswordReset', ['email' => $tokenValid[0]->email]);

            $json = [
                'type' => 'success',
                'redirect' => url('/'),
                'message' => 'Password have successfully updated.'
            ];

            return $json;
        }

        $json = [
            'type' => 'error',
            'redirect' => url('/'),
            'message' => 'The password reset link is no longer valid.'
        ];

        return $json;
    }

    protected function validateResetPassword($request)
    {
        $input = $request->all();
        $rules = [
            'email' => 'required|email',
            'token' => 'required',
            'password' => 'required|min:6|confirmed',
        ];

        return Validator::make($input, $rules);
    }

    protected function getResetInfo($token)
    {
        $info = $this->userRepo->getResetDataByUser($token);

        if (!empty($info) && count($info) > 0) {
            return $info;
        }

        return false;
    }

    protected function sendPasswordResetLink($input)
    {
        $user = $this->userRepo->checkEmailValid($input['r_email']);

        if (!empty($user) && count($user) > 0) {

            return $this->sendResetEmailLink($input, $user);

//            $json = [
//                'type' => 'success',
//                'redirect' => url('/'),
//                'message' => 'Password reset link has sent.Check your email.'
//            ];
//            return $json;
        }

        $json = [
            'type' => 'error',
            'redirect' => url('/'),
            'message' => 'This email is invalid.'
        ];

        return $json;

    }

    public function sendResetEmailLink($input, $user)
    {
        $resetToken = md5(str_random(25));

        $this->savePasswordResetToken($input['r_email'], $resetToken);

        $changeUrl = url('changePassword?change_token=' . $resetToken);

        $data = view('admin.security.emails.user.change_password_link_email', compact('input', 'user', 'changeUrl'));
        $subject = "Reset your Payment Platform password";
        $message = 'Password reset link has sent.Check your email.';

        // send email with reset link code
        return $this->sendCommonMail($input['r_email'], $data, $subject, $message);
    }
}