<?php

namespace App\Traits;

use App\Helpers\FormatHelper;
use App\Repositories\CommonRepo;
use App\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rule;
use Spatie\Permission\Models\Role;
use Validator;
use DataTables;


//use Yajra\DataTables\Contracts\DataTable;

trait UserTrait
{
    use CommonTrait;

    public function getRoles()
    {
        return Role::get();
    }

    public function checkUserName($request)
    {
        $input = $request->all();
        $edit = isset($input['edit']) ? $input['edit'] : false;
        $result = CommonRepo::checkExist('users', $input['field'], $input['value'], $edit);

        if (!$result) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => 'Sorry! User name already in use.'
            ];

            return $json;
        }

        $json = [
            'type' => 'success',
            'redirect' => '',
            'message' => ''
        ];

        return $json;
    }

    private function userValidation($input)
    {
        $rules = [
            'name' => 'required',
            'email' => ['required', 'email', 'max:255', Rule::unique('users')->whereNull('deleted_at')->whereNull('deleted_at')],
            'password' => 'required|min:6|confirmed',
            //'mac_address' => 'required'
        ];

        return Validator::make($input, $rules);
    }

    private function userValidationUpdate($input, $id)
    {
        $rules = [
            'name' => 'required',
                'email' => ['required', 'email', 'max:255', Rule::unique('users')->ignore($id, 'id')->whereNull('deleted_at')],
            //'password' => 'required|min:6|confirmed',
            //'mac_address' => 'required'
        ];

        return Validator::make($input, $rules);
    }

    private function saveUser($request)
    {
        $input = $request->all();

        $input['password'] = Hash::make($input['password']);
        $input['pin_number'] = FormatHelper::randNumberGenerator(4);

        if( $request->hasFile('image_url')){
            $fileName = $this->insertImage($request->file('image_url'), 'u_file', 'user_profile');
            $input['image_url'] = $fileName;
        }

        DB::beginTransaction();

        try {

            $model = 'App\User';
            $user = CommonRepo::commonStore($model, $input);

            $user->assignRole($input['role']);

            $this->setUserPermissions($user);

            $resetToken = md5(str_random(25));

            $this->savePasswordResetToken($user->email, $resetToken);

            $changeUrl = url('changePassword?change_token=' . $resetToken);

            $data = view('admin.security.emails.user.welcome_email', compact('input', 'user', 'changeUrl'));
            $subject = "Welcome to Payment Platform";
            $message = 'Activation email sent.Check your email.';

            // send email with verification code
            $this->sendCommonMail($input['email'], $data, $subject, $message);

        } catch (\Exception $e) {
            DB::rollback();
            dd($e);
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => 'Sorry! Please try again'
            ];

            return $json;
        }

        DB::commit();

        $json = [
            'type' => 'success',
            'redirect' => url('user'),
            'message' => 'Successfully saved.'
        ];

        return $json;
    }

    protected function setUserPermissions($user)
    {
        $permissions = $user->getPermissionsViaRoles();

        if (!empty($permissions) && count($permissions) > 0) {
            foreach ($permissions as $val) {
                $data = [
                    'permission_id' => $val->id,
                    'model_id' => $user->id,
                    'model_type' => ''
                ];

                CommonRepo::commonStore('App\Models\ModelHasPerm', $data);
            }
        }
    }

    protected function savePasswordResetToken($email, $token)
    {
        $data = [
            'email' => $email,
            'token' => $token
        ];

        CommonRepo::commonStore('App\Models\PasswordReset', $data);
    }

    private function updateUser($request, $id)
    {
        $input = $request->all();

        if(isset($input['remove_image'])){
            $input['image_url'] = '';
        }

        if( $request->hasFile('image_url')){
            $fileName = $this->insertImage($request->file('image_url'), 'u_file', 'user_profile');
            $input['image_url'] = $fileName;
        }

        DB::beginTransaction();

        try {

            $model = 'App\User';
            $whereArray = ['id' => $id];
            //$role = $input['role'];
            //unset($input['role']);
            unset($input['remove_image']);
            CommonRepo::commonUpdate($model, $whereArray, $input);

            //$user = User::find($id);
            //$user->assignRole($role);

        } catch (\Exception $e) {
            DB::rollback();
            //dd($e);
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => 'Sorry! Please try again'
            ];

            return $json;
        }

        DB::commit();

        $json = [
            'type' => 'success',
            'redirect' => url('user'),
            'message' => 'Successfully saved.'
        ];

        return $json;
    }

    public function userInfo($data)
    {
        return DataTables::of($data)
            ->addIndexColumn()
            ->addColumn('action', function ($data) {
                $deleteText = "";
                if (Auth::id() != $data->id) {
                    $deleteText = '<button class="btn btn-outline-danger delete-btn btn-sm" data-delete_url="' . url('user/' . $data->id) . '"><i class="la la-trash-o"></i> </button> ';
                }
               return '<a href="' . url('user/' . $data->id . '/edit') . '" class="btn btn-outline-info btn-complete btn-sm  mr-p5 link-btn"><i class="la la-pencil"></i></a>'.$deleteText;
//                return '<div class="dropdown dropdown-default">
//                          <button class="btn dropdown-toggle" type="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
//                            Action
//                          </button>
//                          <div class="dropdown-menu">
//                            <a class="dropdown-item" href="' . url('user/' . $data->id . '/edit') . '"> Edit </a>' . $deleteText . '
//                          </div>
//                        </div>';

            })
            ->editColumn('status', function ($data) {

                if ($data->status == 1) {
                    $status = '<div class="badge badge-success">Active</div>';
                } else {
                    $status = '<div class="badge badge-danger">Deactivated</div>';
                }
                return $status;
            })
            ->editColumn('is_logged_in', function ($data) {
                if ($data->is_logged_in == 1) {
                    $loginStatus = '<span class="badge badge-success">&nbsp;&nbsp;</span>';
                } else {
                    $loginStatus = '<span class="badge badge-warning">&nbsp;&nbsp;</span>';
                }
                return $loginStatus;
            })
            ->editColumn('pin_number', function ($data) {
                return '<a style="color: #1f23a77a !important;cursor: pointer;" onclick="event.preventDefault();">' . $data->pin_number . '</a>';
            })
            ->editColumn('role', function ($data) {

                return $data->getRoleNames()->implode(' ');

            })
            ->rawColumns(['status', 'action', 'is_logged_in', 'pin_number', 'is_logged_in'])
            ->make(true);
    }

    protected function sendPasswordChangeEmail($input)
    {
        $code = FormatHelper::randNumberGenerator(6);
        $token = md5(str_random(20));
        $user = $this->userRepo->checkUserByUserName($input['email']);
        $dataArray = [
            'user_id' => $user[0]->id,
            'login_code' => $code,
            'email_token' => $token,
        ];

        $this->userRepo->addLoginCode($dataArray);

        $data = view('import.emails.auth.verification_code', compact('input', 'user', 'code'));
        $subject = "User verification code";

        try {

            //Mail::to('pasan@pragicts.com')->send(new SendMail($data, $subject));
            $json = [
                'type' => 'success',
                'redirect' => '',
                'step' => 3,
                'layers' => FormatHelper::getLayers(),
                'message' => 'Email verification code sent.Check your email.'
            ];
            return $json;

        } catch (\Exception $e) {
            dd($e);
            $json = [
                'type' => 'error',
                'redirect' => 'reload',
                'step' => 0,
                'message' => 'Sorry! Please try again.'
            ];

            return $json;
        }
    }

    function insertImage($avatar, $i, $folderName)
    {
        $filename = $i . '_' . md5(microtime()) . '_' . $i . '.' . $avatar->getClientOriginalExtension();

        /**
         * If need we can resize image
         * Image::make($avatar)->resize(300, 300)->save(public_path('uploads/country/' . $filename));
         */
        $path = public_path('admin/uploads/' . $folderName . '/');
        //array_map('unlink', glob($path . "*"));
        if (!\Illuminate\Support\Facades\File::exists($path)) {
            \Illuminate\Support\Facades\File::makeDirectory($path, 0777, true);
        }
        copy($avatar->getRealPath(), public_path('admin/uploads/' . $folderName . '/' . $filename));
//        if ($avatar->getClientOriginalExtension() == 'gif') {
//
//        } else {
//            Image::make($avatar)->save(public_path('admin/uploads/' . $folderName . '/' . $filename));
//        }


        return $filename;
    }

}